class DocElementsController < ApplicationController

  layout "none"

  before_filter :find_doc, :only => [:index, :new, :add, :preview]
  before_filter :find_doc_element, :only => [:edit, :update, :destroy, :move, :show]
  before_filter :authorize
  
  helper :attachments
  include AttachmentsHelper
  
  def new
    @doc_element = DocElement.new
    @doc_element.parent_id = params[:parent_id]
    @doc_element.position = params[:position]
    create_doc_element_values(@doc_element, @doc)
  end
  
  def add
    params_doc_element_values = params[:doc_element][:doc_element_values] != nil ? params[:doc_element][:doc_element_values].clone : nil
    params[:doc_element][:doc_element_values] = [] 
    @doc_element = DocElement.new(params[:doc_element])
    if request.post? 
      @doc_element.doc_id = @doc.id
      @doc_element.user_id = User.current.id
      if @doc_element.save
        attach_files(@doc_element, params[:attachments])
        if params_doc_element_values == nil or update_doc_element_values(params_doc_element_values, @doc_element)
          flash[:notice] = l(:notice_successful_create)
          redirect_to :controller => :docs, :action => :edit, :id => @doc.id, :anchor => "doc_element_toc_#{@doc_element.id}"
        end
      end
    end
  end

  def edit
    if @doc_element.is_section 
      create_doc_element_values(@doc_element, @doc)
    end
  end
  
  def update
    params_doc_element_values = params[:doc_element][:doc_element_values] != nil ? params[:doc_element][:doc_element_values].clone : nil
    params[:doc_element][:doc_element_values] = [] 
    if request.post? and @doc_element.update_attributes(params[:doc_element])
      if params_doc_element_values == nil or update_doc_element_values(params_doc_element_values, @doc_element)
        attach_files(@doc_element, params[:attachments])
        flash[:notice] = l(:notice_successful_update)
        redirect_to :controller => :docs, :action => :edit, :id => @doc.id, :anchor => "doc_element_toc_#{@doc_element.id}"
      end
    end
  end
  
  def destroy
    if @doc_element.lower_item != nil
      anchor = "doc_element_toc_#{@doc_element.lower_item.id}"
    elsif @doc_element.higher_item != nil
      anchor = "doc_element_toc_#{@doc_element.higher_item.id}"
    elsif @doc_element.parent != nil
      anchor = "doc_element_toc_#{@doc_element.parent.id}"
    else
      anchor = "doc_element_toc_end" 
    end
    @doc_element.destroy
    flash[:notice] = l(:notice_successful_delete)
    redirect_to :controller => :docs, :action => :edit, :id => @doc.id, :anchor => anchor
  rescue
    flash[:error] = l(:notice_unable_delete)
    redirect_to :controller => :docs, :action => :edit, :id => @doc.id, :anchor => "doc_element_toc_#{@doc_element.id}"
  end
    
  def move
    case params[:position]
    when "up"
      parent = @doc_element.parent 
      if parent != nil 
        position = parent.position + 1
        @doc_element.parent_id = parent.parent_id
        @doc_element.remove_from_list
        @doc_element.insert_at(position)
        @doc_element.save
      end
    when "down"
      previous = @doc_element.higher_item
      if previous != nil and previous.is_section
        position = previous.children.length + 1
        @doc_element.parent_id = previous.id
        @doc_element.remove_from_list
        @doc_element.insert_at(position)
        @doc_element.save
      end
    when "highest"
      @doc_element.move_to_top
    when "higher"
      @doc_element.move_higher
    when "lower"
      @doc_element.move_lower
    when "lowest"
      @doc_element.move_to_bottom
    end if params[:position]
    redirect_to :controller => :docs, :action => :edit, :id => @doc.id, :anchor => "doc_element_toc_#{@doc_element.id}"
  end

  def show
  end

  def preview
    @doc_element = DocElement.new(params[:doc_element])
    @doc_element.attachments = DocElement.find(params[:doc_element_id]).attachments
    render :action => :show
  end

private

  def find_doc
    @doc = Doc.find(params[:id])
    @project = @doc.project
  rescue ActiveRecord::RecordNotFound
    render_404
  end
  
  def find_doc_element
    @doc_element = DocElement.find(params[:id])
    @doc = @doc_element.doc
    @project = @doc.project
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  def create_doc_element_values(doc_element, doc)
    doc_element.doc_element_values = []
    doc.element_type.element_fields.each do |element_field|
      doc_element_value = DocElementValue.new
      doc_element_value.element_field_id = element_field.id 
      doc_element.doc_element_values << doc_element_value
    end
  end
  
  def update_doc_element_values(params_doc_element_values, doc_element)
    ok = true
    params_doc_element_values.each_pair do |element_field_id, value|
      doc_element_value = DocElementValue.new
      doc_element_value.element_field_id = element_field_id
      doc_element_value.doc_element_id = doc_element.id
      doc_element_value.value = value
      if !(doc_element_value.save)
        ok = false
      end
    end
    ok
  end
    
end
