class DocsController < ApplicationController

  before_filter :find_project, :only => [:index, :add, :copy, :update_documents]
  before_filter :find_doc, :only => [:show, :show_traceability, :export_html, :export_html_traceability, :edit, :destroy, :toc, :content]
  before_filter :authorize

  def index
    @docs = Doc.find(:all, :conditions => "project_id = #{@project.id}").sort
  end

  def add
    @doc = Doc.new(params[:doc])
    if request.post? 
      @doc.project_id = @project.id
      @doc.user_id = User.current.id
      if @doc.save
        flash[:notice] = l(:notice_successful_create)
        redirect_to :action => :index, :id => @project.id
      end
    end
  end

  def copy
    if request.post?
      copy_doc(@project, Doc.find(params[:documents_select]))
      redirect_to :action => :index, :id => @project.id
    else
      @projects = []
      Project.find(:all).each do |project|
        docs = Doc.find(:all, :conditions => "project_id = #{project.id}")
        @projects << {:project => project, :docs => docs, :selected => (project.id == @project.id)}
      end
      @root_projects = Project.find(:all, :conditions => "parent_id IS NULL AND status = 1")
      @project_id = params[:project_id] || @project.id
    end
  end
  
  def update_documents
    @project_id = params[:project_id]
    @documents = Doc.find(:all, :conditions => "project_id = #{@project_id}")
    render :layout => false
  end
  
  def show
    @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
  end
  
  def show_traceability
    @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
    @traceability_docs = TraceabilityDoc.find(:all, :conditions => "traced_id = #{@doc.id}")
  end
  
  def export_html
    @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
    send_data(render(:layout => false), :type => "text/html", :filename => "#{@doc.name}.html")
  end

  def export_html_traceability
    @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
    @traceability_docs = TraceabilityDoc.find(:all, :conditions => "traced_id = #{@doc.id}")
    send_data(render(:layout => false), :type => "text/html", :filename => "#{l(:traceability_docs_title)} - #{@doc.name}.html")
  end
  
  def edit
    if request.post? and @doc.update_attributes(params[:doc])
      flash[:notice] = l(:notice_successful_update)
      redirect_to :action => :index, :id => @doc.project.id
    else
      @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
    end
  end
  
  def destroy
    project_id = @doc.project.id
    @doc.destroy
    flash[:notice] = l(:notice_successful_delete)
    redirect_to :action => :index, :id => project_id
  rescue
    flash[:error] = l(:notice_unable_delete)
    redirect_to :action => :index, :id => project_id
  end
    
private

  def find_project
    @project = Project.find(params[:id])
  rescue ActiveRecord::RecordNotFound
    render_404
  end
  
  def find_doc
    @doc = Doc.find(params[:id])
    @project = @doc.project
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  def copy_doc(project, doc)
    new_doc = Doc.new
    new_doc.project_id = project.id
    new_doc.user_id = User.current.id
    new_doc.element_type_id = doc.element_type_id
    new_doc.name = copy_doc_name(project, doc.name)
    new_doc.description = doc.description
    new_doc.version = doc.version
    if new_doc.save
      ok = true
      DocElement.find(:all, :conditions => "doc_id = #{doc.id} AND parent_id IS NULL").each do |doc_element|
        ok = false unless copy_doc_element(doc, doc_element, new_doc)
      end
      new_doc.destroy unless ok
    end
  end
  
  def copy_doc_element(doc, doc_element, new_doc, new_parent_id = nil)
    new_doc_element = DocElement.new
    new_doc_element.doc_id = new_doc.id
    new_doc_element.parent_id = new_parent_id
    new_doc_element.user_id = User.current.id
    new_doc_element.name = doc_element.name
    new_doc_element.content = doc_element.content
    new_doc_element.is_section = doc_element.is_section
    new_doc_element.position = doc_element.position
    ok = true
    if new_doc_element.save
      DocElementValue.find(:all, :conditions => "doc_element_id = #{doc_element.id}").each do |doc_element_value|
        new_doc_element_value = DocElementValue.new
        new_doc_element_value.doc_element_id = new_doc_element.id
        new_doc_element_value.element_field_id = doc_element_value.element_field_id
        new_doc_element_value.value = doc_element_value.value
        ok = false unless new_doc_element_value.save
      end
      ok = false unless copy_attachments(doc_element, new_doc_element)
      DocElement.find(:all, :conditions => "doc_id = #{doc.id} AND parent_id = #{doc_element.id}").each do |child_doc_element|
        ok = false unless copy_doc_element(doc, child_doc_element, new_doc, new_doc_element.id)
      end
    else
      ok = false
    end
    ok
  end

  def copy_attachments(doc_element, new_doc_element)
    ok = true
    doc_element.attachments.each do |attachment|
      new_attachment = attachment.clone
      new_attachment.container_id = new_doc_element.id
      new_attachment.downloads = 0
      new_attachment.author_id = User.current.id
      new_attachment.disk_filename = generate_disk_filename(attachment.filename)
      FileUtils.cp(attachment.diskfile, new_attachment.diskfile)
      ok = false unless new_attachment.save
    end
    ok
  end

  def generate_disk_filename(filename)
    df = DateTime.now.strftime("%y%m%d%H%M%S") + "_"
    if filename =~ %r{^[a-zA-Z0-9_\.\-]*$}
      df << filename
    else
      df << Digest::MD5.hexdigest(filename)
      # keep the extension if any
      df << $1 if filename =~ %r{(\.[a-zA-Z0-9]+)$}
    end
    df
  end
    
  def copy_doc_name(project, name, index = 1)
    copy_name = l(:label_copy_index_of_name, index, name)
    if Doc.find(:all, :conditions => "project_id = #{project.id} AND name = \"#{copy_name}\"").length > 0
      copy_doc_name(project, name, index + 1)
    else
      copy_name
    end
  end
      
end
