class TraceabilityDocsController < ApplicationController

  before_filter :find_project, :only => [:index, :add]
  before_filter :find_traceability_doc, :only => [:show, :edit, :edit_traced, :update_traced, :destroy]
  before_filter :authorize

  def index
    @traceability_docs = TraceabilityDoc.find(:all, :conditions => "project_id = #{@project.id}").sort
  end

  def add
    @traceability_doc = TraceabilityDoc.new(params[:traceability_doc])
    if request.post? 
      @traceability_doc.project_id = @project.id
      @traceability_doc.user_id = User.current.id
      if @traceability_doc.save
        flash[:notice] = l(:notice_successful_create)
        redirect_to :action => :index, :id => @project.id
      end
    else
      docs = Doc.find(:all, :conditions => "project_id = #{@project.id}")
      @tracers = docs.collect{|element_type| [element_type.name, element_type.id]}
      @traceds = @tracers
    end
  end

  def show
    @root_traced_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@traceability_doc.traced.id} AND parent_id IS NULL")
  end
  
  def edit
    if request.post? and @traceability_doc.update_attributes(params[:traceability_doc])
      flash[:notice] = l(:notice_successful_update)
      redirect_to :action => :index, :id => @traceability_doc.project.id
    else
      docs = Doc.find(:all, :conditions => "project_id = #{@project.id}")
      @tracers = docs.collect{|element_type| [element_type.name, element_type.id]}
      @traceds = @tracers
      @root_tracer_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@traceability_doc.tracer.id} AND parent_id IS NULL")
      @tracer_id = params[:tracer_id]
    end
  end
  
  def edit_traced
    @tracer = DocElement.find(params[:tracer_id])
    @root_traced_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@traceability_doc.traced.id} AND parent_id IS NULL")
    @traceabilities = Traceability.find(:all, :conditions => "traceability_doc_id = #{@traceability_doc.id} AND tracer_id = #{@tracer.id}")
    render :layout => false
  end
  
  def update_traced
    @tracer = DocElement.find(params[:tracer_id])
    traceds_to_remove = Traceability.find(:all, :conditions => "traceability_doc_id = #{@traceability_doc.id} AND tracer_id = #{@tracer.id}")
    traceds_to_add = []
    if params[:traced] != nil
      params[:traced].each do |traced_id|
        if traceds_to_remove.collect {|t| t.traced_id.to_s}.include?(traced_id)
          traceds_to_remove.delete_if {|t| t.traced_id.to_s == traced_id}
        else
          traceds_to_add << traced_id
        end
      end
    end
    traceds_to_remove.each do |traced_to_remove|
      traced_to_remove.destroy
    end
    traceds_to_add.each do |traced_to_add|
      traceability = Traceability.new
      traceability.traceability_doc_id = @traceability_doc.id
      traceability.traced_id = traced_to_add
      traceability.tracer_id = @tracer.id
      traceability.save
    end
    redirect_to :action => :edit, :id => @traceability_doc.id, :tracer_id => @tracer.id
  end

  def destroy
    @traceability_doc.destroy
    flash[:notice] = l(:notice_successful_delete)
    redirect_to :action => :index, :id => @project.id
  rescue
    flash[:error] = l(:notice_unable_delete)
    redirect_to :action => :index, :id => @project.id
  end
    
private

  def find_project
    @project = Project.find(params[:id])
  rescue ActiveRecord::RecordNotFound
    render_404
  end
  
  def find_traceability_doc
    @traceability_doc = TraceabilityDoc.find(params[:id])
    @project = @traceability_doc.project
  rescue ActiveRecord::RecordNotFound
    render_404
  end
  
end
