module DocElementsHelper

  def doc_element_value_tag(doc_element_value)	
    element_field = doc_element_value.element_field
    field_name = "doc_element[doc_element_values][#{element_field.id}]"
    field_id = "doc_element_doc_element_values_#{element_field.id}"
    
    case element_field.field_format
    when "date"
      text_field_tag(field_name, 
                     doc_element_value.value, 
                     :id => field_id, 
                     :size => 10) + 
      image_tag("calendar.png",
                {:id => "#{field_id}_trigger", :class => "calendar-trigger"})
    when "text"
      text_area_tag(field_name, 
                    doc_element_value.value, 
                    :id => field_id, 
                    :rows => 3, 
                    :style => "width: 90%;")
    when "bool"
      check_box_tag(field_name, 
                    "1", 
                    doc_element_value.true?, 
                    :id => field_id) + 
      hidden_field_tag(field_name, "0")
    when "list"
      blank_option = element_field.is_required? ?
                     (element_field.default_value.blank? ? "<option value=\"\">--- #{l(:actionview_instancetag_blank_option)} ---</option>" : "") : 
                     "<option></option>"
      select_tag(field_name, 
                 blank_option + options_for_select(element_field.possible_values, doc_element_value.value), 
                 :id => field_id)
    else
      text_field_tag(field_name, 
                     doc_element_value.value, 
                     :id => field_id)
    end
  end
  
  def doc_element_value_label_tag(doc_element_value)
    content_tag("label", 
                doc_element_value.element_field.name + (doc_element_value.element_field.is_required? ? " <span class=\"required\">*</span>" : ""),
                :for => "doc_element_doc_element_values_#{doc_element_value.element_field.id}",
                :class => (doc_element_value.errors.empty? ? nil : "error" ))
  end
  
  def doc_element_value_with_label_tag(doc_element_value)
    doc_element_value_label_tag(doc_element_value) + doc_element_value_tag(doc_element_value)
  end

  def show_value(doc_element_value)
    return "" unless doc_element_value
    format_value(doc_element_value.value, doc_element_value.element_field.field_format)
  end
  
  def format_value(doc_element_value, attachments)
    return "" unless doc_element_value.value && !doc_element_value.value.empty?
    case doc_element_value.element_field.field_format
    when "date"
      begin; value = format_date(doc_element_value.value.to_date); rescue; value = doc_element_value.value end
    when "bool"
      value = l_YesNo(doc_element_value.value == "1")
    else
      value = doc_element_value.value
    end
    textilizable("*#{doc_element_value.element_field.name}:* #{value}", :attachments => attachments)
  end

  def render_header(doc_element)
    doc_element == nil ? "" : "#{doc_element.parent == nil ? "" : "#{render_header(doc_element.parent)}."}#{doc_element.position}"
  end
  
end
