module DocsHelper

  include DocElementsHelper
  
  def tree_select(categories, model, name, selected = nil, level = 0, init = true)
    html = ""
    if init
        html << "<select name=\"#{model}[#{name}]\" id=\"#{model}_#{name}\">\n"
        html << "\t<option value=\"0\""
        html << " selected=\"selected\"" if selected != nil and selected.parent_id == 0
        html << ">Root</option>\n"
    end

    if categories != nil and categories.length > 0
      level += 1
      categories.collect do |cat|
        html << "\t<option value=\"#{cat.id}\" style=\"padding-left:#{level * 10}px\""
        html << ' selected="selected"' if selected != nil and cat.id == selected.parent_id
        html << ">#{cat.name}</option>\n"
        html << tree_select(cat.children, model, name, selected, level, false)
      end
    end
    html << "</select>\n" if init
    return html
  end

  def project_tree_select(projects, current_project_id, selected_id = nil, level = 0, init = true)
    html = ""
    if init
      html << "<select name=\"project_id\" id=\"project_id\" onChange=\"#{remote_function(:update => :documents_select, :url => {:action => :update_documents, :id => current_project_id}, :with => "Form.Element.serialize(this)")};\">\n"
    end
    if projects != nil and projects.length > 0
      projects.collect do |project|
        html << "  <option value=\"#{project.id}\" style=\"padding-left:#{level * 20}px\""
        html << " selected=\"selected\"" if selected_id != nil and project.id == selected_id
        html << ">#{project.name}</option>\n"
        html << project_tree_select(project.children, current_project_id, selected_id, level + 1, false)
      end
    end
    html << "</select>\n" if init
    return html
  end

  def render_toc(doc_elements, editable = true, level = 0, header = "", init = true)
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      if init
        html << "<table class=\"#{editable ? "list" : ""}\">\n"
        if editable
          html << "  <thead>\n"
          html << "    <th width=\"1%\"></th>\n"
          html << "    <th>#{l(:field_name)}</th>\n"
          html << "    <th width=\"25%\">#{l(:button_sort)}</th>\n" if editable
          html << "    <th width=\"15%\"></th>\n" if editable
          html << "  </thead>\n"
        end
        html << "  <tbody>\n"
      end
      index = 1
      doc_elements.collect do |doc_element|
        my_header = "#{header}#{"." if header != ""}#{index}"
        html << "    <tr class=\"#{editable ? cycle("odd", "even") : ""}\">\n"
        html << "      <td>\n"
        html << "        #{h(my_header)}\n"
        html << "      </td>\n"
        html << "      <td style=\"padding-left: #{level * 20}px;\">\n"
        if editable
          javascript = "var toolbar = new jsToolBar($('doc_element_content')); toolbar.draw(); " +
                       "update_doc_element_values_div (); "
          doc_element.doc.element_type.element_fields.each do |element_field|
            if element_field.field_format == "date"
              field_id = "doc_element_doc_element_values_#{element_field.id}"
              javascript +=  "Calendar.setup({inputField : '#{field_id}', ifFormat : '%Y-%m-%d', button : '#{field_id}_trigger' }); "
            end
          end
          visible = "document.getElementById('element_doc_div').style.display = 'inline';"
          if doc_element.is_section
            html << "        #{image_tag("folder.png")}\n"
          else
            html << "        #{DocsHelper.plugin_image_tag("element_type_#{doc_element.doc.element_type.icon}.png")}\n"
          end
        end
        if editable
          html << "        <a name=\"doc_element_toc_#{doc_element.id}\"></a>\n"
          html << "        #{link_to_remote_if_authorized(h(doc_element.name), {:url => {:controller => :doc_elements, :action => :show, :id => doc_element.id}, :update => :element_doc_div}, {:onclick => visible})}\n"
        else
          html << "        #{link_to(h(doc_element.name), :anchor => "doc_element_#{doc_element.id}")}\n"
        end
        html << "      </td>\n"
        if editable
          html << "      <td align=\"center\">\n"
          html << "        #{link_to_if_authorized(image_tag("arrow_to.png",   :alt => l(:label_sort_up)),       {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "up"},      :method => :post, :title => l(:label_sort_up     ))} -\n"
          html << "        #{link_to_if_authorized(image_tag("arrow_from.png", :alt => l(:label_sort_down)),     {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "down"},    :method => :post, :title => l(:label_sort_down   ))} -\n"
          html << "        #{link_to_if_authorized(image_tag("2uparrow.png",   :alt => l(:label_sort_highest)),  {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "highest"}, :method => :post, :title => l(:label_sort_highest))}\n"
          html << "        #{link_to_if_authorized(image_tag("1uparrow.png",   :alt => l(:label_sort_higher)),   {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "higher"},  :method => :post, :title => l(:label_sort_higher ))} -\n"
          html << "        #{link_to_if_authorized(image_tag("1downarrow.png", :alt => l(:label_sort_lower)),    {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "lower"},   :method => :post, :title => l(:label_sort_lower  ))}\n"
          html << "        #{link_to_if_authorized(image_tag("2downarrow.png", :alt => l(:label_sort_lowest)),   {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "lowest"},  :method => :post, :title => l(:label_sort_lowest ))}\n"
          html << "      </td>\n"
          html << "      <td align=\"center\">\n"
          html << "        #{link_to_remote_if_authorized(image_tag("add.png", :alt => l(:button_add)), {:url => {:controller => :doc_elements, :action => :new, :id => @doc.id, :parent_id => doc_element.id, :position => doc_element.children.length + 1}, :update => :element_doc_div, :complete => javascript}, {:style => doc_element.is_section ? "" : "visibility: hidden;", :onclick => visible})}\n"
          html << "        #{link_to_remote_if_authorized(image_tag("edit.png", :alt => l(:button_edit)), {:url => {:controller => :doc_elements, :action => :edit, :id => doc_element.id}, :update => :element_doc_div, :complete => javascript}, {:onclick => visible})}\n"
          html << "        #{link_to_if_authorized(image_tag("delete.png", :alt => l(:button_delete)), {:controller => :doc_elements, :action => :destroy, :id => doc_element.id}, :confirm => l(:text_are_you_sure), :method => :post)}\n"
          html << "      </td>\n"
        end
        html << "    </tr>\n"
        html << render_toc(doc_element.children, editable, level + 1, my_header, false)
        index += 1
      end
      if init
        html << "  </tbody>\n"
        html << "</table>\n"
        html << "<a name=\"doc_element_toc_end\"></a>\n"
        reset_cycle
      end
    end
    return html
  end
  
  def render_doc(doc_elements, level = 0, header = "")
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      index = 1
      doc_elements.collect do |doc_element|
        my_header = "#{header}#{"." if header != ""}#{index}"
        header_level = level > 5 ? 5 : level;
        html << "<div class=\"doc\" style=\"padding-left: #{level * 30}px;\">"
        html << "  <a name=\"doc_element_#{doc_element.id}\"></a>\n"
        html << "  <h#{header_level + 1} style=\"#{header_style} #{header_styles[header_level]}\">#{h(my_header)} #{h(doc_element.name)}</h#{header_level + 1}>\n"
        html << "  #{textilizable(doc_element.content, :attachments => doc_element.attachments)}\n"
        doc_element.doc_element_values.each do |doc_element_value|
          html << "  #{format_value(doc_element_value, doc_element.attachments)}\n"
        end
        html << "</div>\n"
        html << render_doc(doc_element.children, level + 1, my_header)
        index += 1
      end
    end
    return html
  end

  def render_doc_export_html(doc_elements, level = 0)
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      doc_elements.collect do |doc_element|
        header_level = level > 5 ? 5 : level;
        html << "<h#{header_level + 1}>#{h(doc_element.name)}</h#{header_level + 1}>\n"
        html << "#{textilizable(doc_element.content, :attachments => doc_element.attachments, :only_path => false)}\n"
        doc_element.doc_element_values.each do |doc_element_value|
          html << "#{format_value(doc_element_value, doc_element.attachments, :only_path => false)}\n"
        end
        html << render_doc_export_html(doc_element.children, level + 1)
      end
    end
    return html
  end

  def render_all_traceabilities(doc, doc_elements, traceability_docs, level = 0, header = "", init = true)
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      if init
        percentage = (100 / (traceability_docs.length + 1)).to_i
        html << "<table class=\"list\">\n"
        html << "  <thead>\n"
        html << "    <th width=\"#{percentage}%\" colspan=\"2\">#{l(:field_element_traced)} (#{doc.name})</th>\n"
        traceability_docs.each do |traceability_doc|
          html << "    <th width=\"#{percentage}%\" colspan=\"2\">#{l(:field_element_tracer)} (#{traceability_doc.tracer.name})</th>\n"
        end
        html << "  </thead>\n"
        html << "  <tbody>\n"
      end
      index = 1
      doc_elements.collect do |doc_element|
        my_header = "#{header}#{"." if header != ""}#{index}"
        html << "    <tr class=\"#{cycle("odd", "even")}\">\n"
        html << "      <td width=\"1%\">\n"
        html << "        <a name=\"traced_id_#{doc_element.id}\"></a>\n"
        html << "        #{h(my_header)}\n"
        html << "      </td>\n"
        html << "      <td style=\"padding-left: #{level * 20}px;\">\n"
        if doc_element.is_section
          html << "        #{image_tag("folder.png")}\n"
        else
          html << "        #{DocsHelper.plugin_image_tag("element_type_#{doc_element.doc.element_type.icon}.png")}\n"
        end
        if doc_element.is_section
          html << "        #{h(doc_element.name)}\n"
        else
          html << "        #{link_to_if_authorized(h(doc_element.name), {:controller => :docs, :action => :show, :id => doc.id, :anchor => "doc_element_#{doc_element.id}"})}\n"
        end
        html << "      </td>\n"
        traceability_docs.each do |traceability_doc|
          if doc_element.is_section
            html << "      <td align=\"center\">\n"
            html << "        &nbsp;\n"
            html << "      </td>\n"
            html << "      <td>\n"
            html << "        &nbsp;\n"
            html << "      </td>\n"
          else
            traceabilities = Traceability.find(:all, :conditions => "traceability_doc_id = #{traceability_doc.id} AND traced_id = #{doc_element.id}").sort
            html << "      <td width=\"1%\" align=\"center\">\n"
            traceabilities.each do |traceability|
              html << "        #{render_header(traceability.tracer)}\n"
              html << "        <br />\n"
            end
            html << "      </td>\n"
            html << "      <td>\n"
            traceabilities.each do |traceability|
              html << "        #{DocsHelper.plugin_image_tag("element_type_#{traceability.tracer.doc.element_type.icon}.png")}\n"
              html << "        #{link_to_if_authorized(h(traceability.tracer.name), {:controller => :docs, :action => :show, :id => traceability_doc.tracer.id, :anchor => "doc_element_#{traceability.tracer.id}"})}\n"
              html << "        <br />\n"
            end
            html << "      </td>\n"
          end
        end
        html << "    </tr>\n"
        html << render_all_traceabilities(doc, doc_element.children, traceability_docs, level + 1, my_header, false)
        index += 1
      end
      if init
        html << "  </tbody>\n"
        html << "</table>\n"
        html << "<a name=\"tracer_end\"></a>\n"
        reset_cycle
      end
    end
    return html
  end
  
  def render_all_traceabilities_export_html(doc, doc_elements, traceability_docs, level = 0, header = "", init = true)
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      if init
        percentage = (100 / (traceability_docs.length + 1)).to_i
        html << "<table class=\"list\">\n"
        html << "  <thead>\n"
        html << "    <th width=\"#{percentage}%\" colspan=\"2\">#{l(:field_element_traced)} (#{doc.name})</th>\n"
        traceability_docs.each do |traceability_doc|
          html << "    <th width=\"#{percentage}%\" colspan=\"2\">#{l(:field_element_tracer)} (#{traceability_doc.tracer.name})</th>\n"
        end
        html << "  </thead>\n"
        html << "  <tbody>\n"
      end
      index = 1
      doc_elements.collect do |doc_element|
        my_header = "#{header}#{"." if header != ""}#{index}"
        html << "    <tr>\n"
        html << "      <td width=\"1%\">\n"
        html << "        #{h(my_header)}\n"
        html << "      </td>\n"
        html << "      <td style=\"padding-left: #{level * 20}px;\">\n"
        html << "        #{h(doc_element.name)}\n"
        html << "      </td>\n"
        traceability_docs.each do |traceability_doc|
          if doc_element.is_section
            html << "      <td>\n"
            html << "        &nbsp;\n"
            html << "      </td>\n"
            html << "      <td>\n"
            html << "        &nbsp;\n"
            html << "      </td>\n"
          else
            traceabilities = Traceability.find(:all, :conditions => "traceability_doc_id = #{traceability_doc.id} AND traced_id = #{doc_element.id}").sort
            html << "      <td width=\"1%\">\n"
            traceabilities.each do |traceability|
              html << "        #{render_header(traceability.tracer)}\n"
              html << "        <br />\n"
            end
            html << "      </td>\n"
            html << "      <td>\n"
            traceabilities.each do |traceability|
              html << "        #{h(traceability.tracer.name)}\n"
              html << "        <br />\n"
            end
            html << "      </td>\n"
          end
        end
        html << "    </tr>\n"
        html << render_all_traceabilities_export_html(doc, doc_element.children, traceability_docs, level + 1, my_header, false)
        index += 1
      end
      if init
        html << "  </tbody>\n"
        html << "</table>\n"
      end
    end
    return html
  end
  
private
  
  def header_style
    "border: 0px; margin: 0px; padding: 20px 0px 10px 0px;"
  end
  
  def header_styles
    {
      0 => "font-size: 24px;",
      1 => "font-size: 20px;",
      2 => "font-size: 17px;",
      3 => "font-size: 15px;",
      4 => "font-size: 14px;",
      5 => "font-size: 13px;"
    }
  end
  
end
