module TraceabilityDocsHelper

  include DocElementsHelper
  
  def render_tracer(doc_elements, traceability_doc, level = 0, header = "", init = true)
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      if init
        html << "<table class=\"list\">\n"
        html << "  <thead>\n"
        html << "    <th width=\"1%\"></th>\n"
        html << "    <th>#{l(:field_name)}</th>\n"
        html << "    <th width=\"15%\">#{l(:field_tracer_count)}</th>\n"
        html << "  </thead>\n"
        html << "  <tbody>\n"
      end
      index = 1
      doc_elements.collect do |doc_element|
        my_header = "#{header}#{"." if header != ""}#{index}"
        tracer_count = Traceability.find(:all, :conditions => "traceability_doc_id = #{traceability_doc.id} AND tracer_id = #{doc_element.id}").length
        visible = "document.getElementById('traced_div').style.display = 'inline';"
        html << "    <tr class=\"#{cycle("odd", "even")}\">\n"
        html << "      <td>\n"
        html << "        <a name=\"tracer_id_#{doc_element.id}\"></a>\n"
        html << "        #{h(my_header)}\n"
        html << "      </td>\n"
        html << "      <td style=\"padding-left: #{level * 20}px;\">\n"
        if doc_element.is_section
          html << "        #{image_tag("folder.png")}\n"
        else
          html << "        #{DocsHelper.plugin_image_tag("element_type_#{doc_element.doc.element_type.icon}.png")}\n"
        end
        if doc_element.is_section
          html << "        #{h(doc_element.name)}\n"
        else
          html << "        #{link_to_remote_if_authorized(h(doc_element.name), {:url => {:action => :edit_traced, :id => traceability_doc.id, :tracer_id => doc_element.id}, :update => :traced_div}, {:onclick => visible})}\n"
        end
        html << "      </td>\n"
        html << "      <td align=\"center\">\n"
        if doc_element.is_section
          html << "        &nbsp;\n"
        else
          html << "        #{tracer_count}\n"
        end
        html << "      </td>\n"
        html << "    </tr>\n"
        html << render_tracer(doc_element.children, traceability_doc, level + 1, my_header, false)
        index += 1
      end
      if init
        html << "  </tbody>\n"
        html << "</table>\n"
        html << "<a name=\"tracer_end\"></a>\n"
        reset_cycle
      end
    end
    return html
  end
  
  def render_traced(doc_elements, traceability_doc, tracer, f, level = 0, header = "", init = true)
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      if init
        html << "<table class=\"list\">\n"
        html << "  <thead>\n"
        html << "    <th width=\"1%\"></th>\n"
        html << "    <th width=\"1%\"></th>\n"
        html << "    <th>#{l(:field_name)}</th>\n"
        html << "    <th width=\"15%\">#{l(:field_traced_count)}</th>\n"
        html << "  </thead>\n"
        html << "  <tbody>\n"
      end
      index = 1
      doc_elements.collect do |doc_element|
        my_header = "#{header}#{"." if header != ""}#{index}"
        tracer_count = Traceability.find(:all, :conditions => "traceability_doc_id = #{traceability_doc.id} AND traced_id = #{doc_element.id}").length
        checked = Traceability.find(:all, :conditions => "traceability_doc_id = #{traceability_doc.id} AND traced_id = #{doc_element.id} AND tracer_id = #{tracer.id}").length > 0
        html << "    <tr class=\"#{cycle("odd", "even")}\">\n"
        html << "      <td>\n"
        if doc_element.is_section
          html << "        &nbsp;\n"
        else
          html << "        <input type=checkbox name=\"traced[]\" value=\"#{doc_element.id}\" #{checked ? "checked" : ""}>\n"
        end
        html << "      </td>\n"
        html << "      <td>\n"
        html << "        #{h(my_header)}\n"
        html << "      </td>\n"
        html << "      <td style=\"padding-left: #{level * 20}px;\">\n"
        if doc_element.is_section
          html << "        #{image_tag("folder.png")}\n"
        else
          html << "        #{DocsHelper.plugin_image_tag("element_type_#{doc_element.doc.element_type.icon}.png")}\n"
        end
        if doc_element.is_section
          html << "        #{h(doc_element.name)}\n"
        else
          html << "        #{link_to_remote_if_authorized(h(doc_element.name), {:url => {:controller => :traceability_docs, :action => :edit_traced, :id => traceability_doc.id, :tracer_id => doc_element.id}, :update => :traced_div}, {})}\n"
        end
        html << "      </td>\n"
        html << "      <td align=\"center\">\n"
        if doc_element.is_section
          html << "        &nbsp;\n"
        else
          html << "        #{tracer_count}\n"
        end
        html << "      </td>\n"
        html << "    </tr>\n"
        html << render_traced(doc_element.children, traceability_doc, tracer, f, level + 1, my_header, false)
        index += 1
      end
      if init
        html << "  </tbody>\n"
        html << "</table>\n"
        html << "<a name=\"tracer_end\"></a>\n"
        reset_cycle
      end
    end
    return html
  end

  def render_traceability(doc_elements, traceability_doc, level = 0, header = "", init = true)
    html = ""
    if doc_elements != nil and doc_elements.length > 0
      if init
        html << "<table class=\"list\">\n"
        html << "  <thead>\n"
        html << "    <th width=\"50%\" colspan=\"2\">#{l(:field_element_traced)} (#{traceability_doc.traced.name})</th>\n"
        html << "    <th width=\"50%\" colspan=\"2\">#{l(:field_element_tracer)} (#{traceability_doc.tracer.name})</th>\n"
        html << "  </thead>\n"
        html << "  <tbody>\n"
      end
      index = 1
      doc_elements.collect do |doc_element|
        my_header = "#{header}#{"." if header != ""}#{index}"
        html << "    <tr class=\"#{cycle("odd", "even")}\">\n"
        html << "      <td width=\"1%\">\n"
        html << "        <a name=\"traced_id_#{doc_element.id}\"></a>\n"
        html << "        #{h(my_header)}\n"
        html << "      </td>\n"
        html << "      <td style=\"padding-left: #{level * 20}px;\">\n"
        if doc_element.is_section
          html << "        #{image_tag("folder.png")}\n"
        else
          html << "        #{DocsHelper.plugin_image_tag("element_type_#{doc_element.doc.element_type.icon}.png")}\n"
        end
        if doc_element.is_section
          html << "        #{h(doc_element.name)}\n"
        else
          html << "        #{link_to_if_authorized(h(doc_element.name), {:controller => :docs, :action => :show, :id => traceability_doc.traced.id, :anchor => "doc_element_#{doc_element.id}"})}\n"
        end
        html << "      </td>\n"
        if doc_element.is_section
          html << "      <td align=\"center\">\n"
          html << "        &nbsp;\n"
          html << "      </td>\n"
          html << "      <td>\n"
          html << "        &nbsp;\n"
          html << "      </td>\n"
        else
          html << "      <td width=\"1%\" align=\"center\">\n"
          doc_element.tracers.each do |tracer|
            html << "        #{render_header(tracer)}\n"
            html << "        <br />\n"
          end
          html << "      </td>\n"
          html << "      <td>\n"
          doc_element.tracers.each do |tracer|
            html << "        #{DocsHelper.plugin_image_tag("element_type_#{tracer.doc.element_type.icon}.png")}\n"
            html << "        #{link_to_if_authorized(h(tracer.name), {:controller => :docs, :action => :show, :id => traceability_doc.tracer.id, :anchor => "doc_element_#{tracer.id}"})}\n"
            html << "        <br />\n"
          end
          html << "      </td>\n"
        end
        html << "    </tr>\n"
        html << render_traceability(doc_element.children, traceability_doc, level + 1, my_header, false)
        index += 1
      end
      if init
        html << "  </tbody>\n"
        html << "</table>\n"
        html << "<a name=\"tracer_end\"></a>\n"
        reset_cycle
      end
    end
    return html
  end
  
end
