class ElementField < ActiveRecord::Base
  
  has_many :element_type_element_fields, :dependent => :destroy
  has_many :element_types, :through => :element_type_element_fields
  serialize :possible_values
  
  FIELD_FORMATS = { "string" => { :name => :label_string, :order => 1 },
                    "text" => { :name => :label_text, :order => 2 },
                    "int" => { :name => :label_integer, :order => 3 },
                    "float" => { :name => :label_float, :order => 4 },
                    "list" => { :name => :label_list, :order => 5 },
              "date" => { :name => :label_date, :order => 6 },
              "bool" => { :name => :label_boolean, :order => 7 }
  }.freeze

  validates_presence_of :name
  validates_uniqueness_of :name
  validates_length_of :name, :maximum => 30
  validates_inclusion_of :field_format, :in => FIELD_FORMATS.keys
    
  def initialize(attributes = nil)
    super
    self.possible_values ||= []
  end
  
  def before_validation
    self.possible_values = self.possible_values.collect{|v| v unless v.empty?}.compact
    true
  end
  
  def validate
    if self.field_format == "list"
      errors.add(:possible_values, :activerecord_error_blank) if self.possible_values.nil? || self.possible_values.empty?
      errors.add(:possible_values, :activerecord_error_invalid) unless self.possible_values.is_a? Array
    end
    # v = CustomValue.new(:custom_field => self.clone, :value => default_value, :customized => nil)
    # v.custom_field.is_required = false
    # errors.add(:default_value, :activerecord_error_invalid) unless v.valid?
  end

  def to_s
    name
  end
  
  def <=>(element_type)
    self.name.downcase <=> element_type.name.downcase
  end

end
